/*
 * @(#) CodeLoader.java	1.7 00/08/01
 *
 * Copyright (c) 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.compiler;

import pnuts.lang.Pnuts;
import java.security.ProtectionDomain;

/**
 * This class is used with JDK1.2 or higher, to define classes from bytecode.
 * The resulting class is in the same protection domain of pnuts.lang.Pnuts class.
 */
class CodeLoader extends ClassLoader {

    private final static boolean DEBUG = false;
    private int count = 0;
    private ProtectionDomain domain;

    CodeLoader(){
	init();
    }

    CodeLoader(ClassLoader loader){
	super(loader);
	init();
    }

    void init(){
	this.domain = getClass().getProtectionDomain();
    }

    protected Class findClass(String name) throws ClassNotFoundException {
	ClassLoader parent = getParent();
	try {
	    if (parent != null){
		return parent.loadClass(name);
	    }
	} catch (ClassNotFoundException e){
	}
	return findSystemClass(name);
    }

    /**
     * Resolve a class
     */
    void resolve(Class c){
	try{
	    resolveClass(c);
	} catch (VerifyError ve){
	    throw ve;
	}
    }

    /**
     * Defines a class from a byte code array.
     * This method can be called until seal() method is called, 
     */
    Class define(String name, byte data[], int offset, int length){
	if (DEBUG){
	    System.out.println(name + ", " + data.length + ", " + offset + ", " + length);
	}
	return defineClass(name, data, offset, length, domain);
    }

    synchronized int nextCount(){
	return count++;
    }
}
