/*
 * @(#) DynamicRuntime.java	1.8 00/04/19
 *
 * Copyright (c) 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.compiler;

import java.lang.reflect.Method;
import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;
import java.lang.reflect.InvocationTargetException;
import pnuts.lang.Runtime;
import pnuts.lang.Context;
import pnuts.lang.PnutsException;
import pnuts.lang.PnutsFunction;
import pnuts.lang.Pnuts;
import pnuts.lang.AbstractData;
import pnuts.lang.Cache;
import java.util.HashMap;
import java.security.*;

public class DynamicRuntime extends Runtime {
    private final static boolean DEBUG = false;
    private static CodeLoader codeLoader = null;
    private static Runtime _rt = null;

    /**
     * call a method
     */
    public static Object callMethod(Context context,
				    Class c,
				    String name,
				    Object args[],
				    Class types[],
				    Object target)
    {
	if (target instanceof AbstractData){
	    return ((AbstractData)target).invoke(name, args, context);
	}
	try {
	    ClassLoader cl = codeLoader;
	    final Runtime rt = getRuntime(context);
	    if (rt != _rt || cl == null){
		    
		if (rt != null){
		    cl = getClassLoader(rt.getClass());
		}
		if (!(cl instanceof CodeLoader)){
		    if (DEBUG){
			System.out.println("createCodeLoader");
		    }
		    cl = Compiler.createCodeLoader(Pnuts.getClassLoader(context), true);
		}
		_rt = rt;
		codeLoader = (CodeLoader)cl;
	    }
	    return callMethod(c, name, args, types, target, (CodeLoader)cl);
	} catch (InvocationTargetException e1){
	    Throwable t = e1.getTargetException();
	    if (t instanceof PnutsException){
		throw (PnutsException)t;
	    } else {
	        throw new PnutsException(t, context);
	    }
	} catch (NoMemberFoundException e2){
	    throw new PnutsException("method.notFound",
				     new Object[]{name, c.getName(), Pnuts.format(args)}, context);
	} catch (PnutsException e3){
	    throw e3;
	} catch (Throwable e4){
	    throw new PnutsException(e4, context);
	}
    }

    public static Object callMethod(Class c,
				    String name,
				    Object args[],
				    Class types[],
				    Object target)
	throws IllegalAccessException,
	       IllegalArgumentException,
	       InvocationTargetException
    {
	CodeLoader cl = codeLoader;
	if (cl != null){
	    return callMethod(c, name, args, types, target, cl);
	} else {
	    ClassLoader l = getClassLoader(c);
	    /*
	    if (l instanceof CodeLoader){
		return callMethod(c, name, args, types, target, (CodeLoader)l);
	    } else {
		return callMethod(c, name, args, types, target, Compiler.createCodeLoader(l, true));
	    }
	    */
	    return callMethod(c, name, args, types, target, Compiler.createCodeLoader(l, true));
	}
    }

    static Object callMethod(Class c,
			     String name,
			     Object args[],
			     Class types[],
			     Object target,
			     CodeLoader codeLoader)
	throws IllegalAccessException,
	       IllegalArgumentException,
	       InvocationTargetException
    {
	ProxyCache methodCache = null;

	if (c.isArray()){
	    c = Object.class;  // This line is needed for JavaStation
	}

	ProxyCache[] m = methodFinder.getMethods(c, name);
	if (m == null){
	    return Runtime.callMethod(c, name, args, types, target);
	}

	int count = 0;
	int min = Integer.MAX_VALUE;
	Stack methods = new Stack();
	int nargs = args.length;
    cand:
	for (int i = 0; i < m.length; i++){
	    Class p[] = m[i].paramTypes;
	    if (p.length != nargs){
		continue;
	    }
	    count = 0;
	    for (int j = 0; j < p.length; j++){
		Class pj = p[j];
		if (types != null){
		    Class tj = types[j];
		    if (tj != null && pj != tj && !pj.isAssignableFrom(tj)){
			continue cand;
		    }
		}
		int t = matchType(pj, args[j]);
		if (t < 0){
		    continue cand;
		}
		count += t;
	    }
	    if (count > min){
		continue;
	    }

	    if ((m[i].type == ProxyCache.STATIC) != (target == null)){
		continue;
	    }

	    if (count < min){
		methods.removeAllElements();
		methods.push(m[i]);
		min = count;
	    } else if (count == min){
		methods.push(m[i]);
		if (DEBUG){
		    System.out.println("push " + m[i]);
		}
	    }
	}

	Class clazz = c;
	out:
	while (clazz != null){
	    int size = methods.size();
	    for (int i = 0; i < size; i++){
		methodCache = (ProxyCache)methods.pop();
		if (methodCache.declaringClass == clazz){
		    break out;
		}
	    }
	    clazz = clazz.getSuperclass();
	}

	if (methodCache != null){
	    if (nargs > 0){
		Class p[] = methodCache.paramTypes;
		for (int j = 0; j < p.length; j++){
		    Class pj = p[j];
		    if (pj.isArray() && args[j] != null && isArray(args[j])){
			if (!pj.isInstance(args[j])){
			    Class compType = getBottomType(pj);
			    args[j] = transform(compType, args[j]);
			}
		    }
		}
	    }
	    if (methodCache.targetClass == null){
		return Runtime.callMethod(c, name, args, types, target);// could be more efficient
	    }

	    if (methodCache.proxy == null){
		methodCache.proxy =
		    DynamicProxyFactory.makeProxy(name,
						  methodCache.targetClass,
						  methodCache.returnType,
						  methodCache.paramTypes,
						  methodCache.type,
						  codeLoader);
	    }
	    try {
		if (nargs == 0){
		    return methodCache.proxy.invoke(target);
		} else {
		    return methodCache.proxy.invoke(target, args);
		}
	    } catch (NoClassDefFoundError err){
		if (DEBUG){
		    System.out.println(err + " , " + codeLoader);
		}
		ClassLoader l = getClassLoader(c);
		if (codeLoader == l){
		    if (DEBUG){
			System.out.println("use reflection");
		    }
		    methodFinder.useReflection(c);
		    return Runtime.callMethod(c, name, args, types, target);
		} else {
		    methodCache.proxy = null;
		    return callMethod(c, name, args, types, target, Compiler.createCodeLoader(l, true));
		}
	    } catch (PnutsException pex){
		throw pex;
	    } catch (Throwable t){
		if (t instanceof ClassNotFoundException){
		    if (DEBUG){
			System.out.println(t);
		    }
		    if (DEBUG){
			System.out.println("use reflection");
		    }
		    methodFinder.useReflection(c);
		    return Runtime.callMethod(c, name, args, types, target);
		} else {
		    throw new PnutsException(t);
		}
	    }
	} else {
	    throw new NoMemberFoundException();
	}
    }

    public static Object newInstance(Context context, Class clazz, Object[] args, Class[] casts){
	if (clazz.isPrimitive()){
	    return Runtime.primitive(context, clazz, args[0], true);
	} else if (clazz.isArray()){
	    return Runtime.cast(context, clazz, args[0], true);
	} else {
	    return callConstructor(context, clazz, args, casts);
	}
    }

    public static Object callConstructor(Context context, Class c, Object args[], Class types[]){
	try {
	    ClassLoader cl = codeLoader;
	    final Runtime rt = getRuntime(context);
	    if (rt != _rt || cl == null){
		cl = null;
		if (rt != null){
		    cl = getClassLoader(getRuntime(context).getClass());
		}
		if (!(cl instanceof CodeLoader)){
		    if (DEBUG){
			System.out.println("createCodeLoader");
		    }
		    cl = Compiler.createCodeLoader(Pnuts.getClassLoader(context), true);
		}
		_rt = rt;
		codeLoader = (CodeLoader)cl;
	    }
	    return callConstructor(c, args, types, (CodeLoader)cl);

	} catch (InvocationTargetException e1){
	    Throwable t = ((InvocationTargetException)e1).getTargetException();
	    if (t instanceof PnutsException){
		throw (PnutsException)t;
	    } else {
		throw new PnutsException(t, context);
	    }
	} catch (NoMemberFoundException e2){
	    throw new PnutsException("constructor.notFound", new Object[]{c, Pnuts.format(args)}, context);
	} catch (PnutsException e3){
	    throw e3;
	} catch (Throwable e4){
	    throw new PnutsException(e4, context);
	}
    }

    /**
     * Call a constructor
     *
     * @param c the class of constructor
     * @param args the paramters
     * @return the created instance
     */
    public static Object callConstructor(Class c, Object args[])
	throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, InstantiationException
    {
	return callConstructor(c, args, null);
    }

    /**
     * Call a constructor
     *
     * @param c the class of constructor
     * @param args the paramters
     * @param types the types of the formal arguments
     * @return the created instance
     */
    public static Object callConstructor(Class c, Object args[], Class types[])
	throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, InstantiationException
    {
	return callConstructor(c, args, types, Compiler.createCodeLoader(getClassLoader(c), true));
    }

    public static Object callConstructor(Class c, Object args[], Class types[], CodeLoader codeLoader)
	throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, InstantiationException
    {
	ProxyCache cs[] = methodFinder.getConstructors(c);
	ProxyCache cons = null;

	if (cs == null){
	    return Runtime.callConstructor(c, args, types);
	}

	int count = 0;
	int min = Integer.MAX_VALUE;
    cand:
	for (int i = 0; i < cs.length; i++){
	    Class p[] = cs[i].paramTypes;
	    if (p.length != args.length){
		continue;
	    }
	    count = 0;
	    for (int j = 0; j < p.length; j++){
		Class pj = p[j];
		if (types != null){
		    Class tj = types[j];
		    if (tj != null && pj != tj && !pj.isAssignableFrom(tj)){
			continue cand;
		    }
		}
		int t = matchType(pj, args[j]);
		if (t < 0){
		    continue cand;
		}
		count += t;
	    }
	    if (count < min){
		min = count;
		cons = cs[i];
	    }
	}
	if (cons != null){
	    Class p[] = cons.paramTypes;
	    for (int j = 0; j < p.length; j++){
		Class pj = p[j];
		if (pj.isArray() && args[j] != null && isArray(args[j])){
		    if (!pj.isInstance(args[j])){
			Class compType = getBottomType(pj);
			args[j] = transform(compType, args[j]);
		    }
		}
	    }
	    if (cons.proxy == null){
		cons.proxy =
		    DynamicProxyFactory.makeProxy("<init>",
						  cons.declaringClass,
						  cons.returnType,
						  cons.paramTypes,
						  ProxyCache.CONSTRUCTOR,
						  codeLoader);
	    }
	    try {
		if (args.length == 0){
		    return cons.proxy.invoke(null);
		} else {
		    return cons.proxy.invoke(null, args);
		}
	    } catch (NoClassDefFoundError err){
		if (DEBUG){
		    System.out.println(err);
		}
		return callConstructor(c, args, types, Compiler.createCodeLoader(getClassLoader(c), true));
	    } catch (PnutsException pex){
		throw pex;
	    } catch (Throwable t){
		/* jdk1.1.8 throws ClassNotFoundException instead of NoClassDefFoundError */
		if (DEBUG){
		    System.out.println(t);
		}
		if (t instanceof ClassNotFoundException){
		    methodFinder.useReflection(c);
		    return Runtime.callConstructor(c, args, types);
		} else {
		    throw new PnutsException(t);
		}
	    }
	} else {
	    throw new NoMemberFoundException();
	}
    }

    static class ProxyCache {
	Class[] paramTypes;
	Class declaringClass;
	Class targetClass;
	Class returnType;
	int type;
	DynamicProxy proxy;
	final static int DEFAULT = 0;
	final static int STATIC = 1;
	final static int CONSTRUCTOR = 2;

	ProxyCache(Method method, Class clazz){
	    this.paramTypes = method.getParameterTypes();
	    this.declaringClass = method.getDeclaringClass();
	    this.returnType = method.getReturnType();
	    this.type = Modifier.isStatic(method.getModifiers()) ? STATIC : DEFAULT;

 	    if (Modifier.isPublic(clazz.getModifiers())){
		this.targetClass = clazz;
 	    } else {
		this.targetClass = findCallableClass(clazz,
						     method.getName(),
						     method.getParameterTypes());
	    }
	}

	ProxyCache(Constructor cons){
	    this.paramTypes = cons.getParameterTypes();
	    this.declaringClass = cons.getDeclaringClass();
	    this.targetClass = cons.getDeclaringClass();
	    this.returnType = void.class;
	    this.type = CONSTRUCTOR;
	}

	private static Class findCallableClass(Class clazz, String name, Class args[]){
	    if (clazz == null){
		return null;
	    }
	    Method method;
	    if (Modifier.isPublic(clazz.getModifiers())){
		try {
		    method = clazz.getMethod(name, args);
		    if (method != null){
			return clazz;
		    }
		} catch (NoSuchMethodException nme){
		}
	    }
	    if (!clazz.isInterface()){
		Class it[] = clazz.getInterfaces();
		for (int i = 0; i < it.length; i++){
		    Class c = findCallableClass(it[i], name, args);
		    if (c != null){
			return it[i];
		    }
		}
		return findCallableClass(clazz.getSuperclass(), name, args);
	    }
	    return null;
	}
    }

    static abstract class MethodFinder {
	public abstract ProxyCache[] getConstructors(Class c);
	public abstract ProxyCache[] getMethods(Class c, String name);
	public abstract void useReflection(Class c);
    }

    static class ClassicMethodFinder extends MethodFinder {
	private static Cache mtab = new Cache(512);
	private static Cache ctab = new Cache(512);
	private static Cache USE_REFLECTION = new Cache(0);

	public void useReflection(Class c){
	    if (DEBUG){
		System.out.println("use reflection for " + c.getName());
	    }
	    mtab.put(c, USE_REFLECTION);
	    ctab.put(c, USE_REFLECTION);
	}

	public ProxyCache[] getMethods(Class c, String name){
	    Cache cache = (Cache)mtab.get(c);
	    if (cache == null){
		cache = new Cache(512);
		mtab.put(c, cache);
	    } else if (cache == USE_REFLECTION){
		return null;
	    }
	    Object v = cache.get(name);
	    if (v instanceof ProxyCache[]){
		return (ProxyCache[])v;
	    } else {
		if (DEBUG){
		    System.out.println(c + ":" + name);
		}
		Method m[] = c.getMethods();
		if (m == null){ // for Bug:4137722
		    throw new NoClassDefFoundError("" + c);
		}
		int j = 0;
		for (int i = 0; i < m.length; i++){
		    String m_name = m[i].getName().intern();
		    if (m_name == name && i >= j){
			m[j] = m[i];
			j++;
		    }
		}
		ProxyCache px[] = new ProxyCache[j];
		for (int i = 0; i < j; i++){
		    px[i] = new ProxyCache(m[i], c);
		}
		cache.put(name, px);
		return px;
	    }
	}

	public ProxyCache[] getConstructors(Class c){
	    Object v = ctab.get(c);
	    if (v instanceof ProxyCache[]){
		return (ProxyCache[])v;
	    } else if (v == USE_REFLECTION){
		return null;
	    } else {
		Constructor con[] = c.getConstructors();
		ProxyCache px[] = new ProxyCache[con.length];
		for (int i = 0; i < con.length; i++){
		    px[i] = new ProxyCache(con[i]);
		}
		ctab.put(c, px);
		return px;
	    }
	}
    }

    static class Java2MethodFinder extends MethodFinder {
	private static HashMap mtab = new HashMap();
	private static HashMap ctab = new HashMap();
	private static java.lang.ref.SoftReference USE_REFLECTION = new java.lang.ref.SoftReference(null);

	public void useReflection(Class c){
	    if (DEBUG){
		System.out.println("use reflection for " + c.getName());
	    }
	    mtab.put(c, USE_REFLECTION);
	    ctab.put(c, USE_REFLECTION);
	}

	public ProxyCache[] getMethods(Class c, String name){
	    java.lang.ref.SoftReference cache = (java.lang.ref.SoftReference)mtab.get(c);
	    HashMap map;
	    if (cache == null || (map = (HashMap)cache.get()) == null){
		map = new HashMap();
		mtab.put(c, new java.lang.ref.SoftReference(map));
	    } else if (cache == USE_REFLECTION){
		return null;
	    }
	    Object v = map.get(name);
	    if (v instanceof ProxyCache[]){
		return (ProxyCache[])v;
	    } else {
		if (DEBUG){
		    System.out.println(c + ":" + name);
		}
		Method m[] = c.getMethods();
		if (m == null){ // for Bug:4137722
		    throw new NoClassDefFoundError("" + c);
		}
		int j = 0;
		for (int i = 0; i < m.length; i++){
		    String m_name = m[i].getName().intern();
		    if (m_name.equals(name) && i >= j){
			m[j] = m[i];
			j++;
		    }
		}
		ProxyCache px[] = new ProxyCache[j];
		for (int i = 0; i < j; i++){
		    px[i] = new ProxyCache(m[i], c);
		}
		map.put(name, px);
		return px;
	    }
	}

	public ProxyCache[] getConstructors(Class c){

	    java.lang.ref.SoftReference cache = (java.lang.ref.SoftReference)ctab.get(c);
	    ProxyCache[] pc;
	    if (cache == USE_REFLECTION){
		return null;
	    } else if (cache == null || (pc = (ProxyCache[])cache.get()) == null){
		Constructor con[] = c.getConstructors();
		ProxyCache px[] = new ProxyCache[con.length];
		for (int i = 0; i < con.length; i++){
		    px[i] = new ProxyCache(con[i]);
		}
		ctab.put(c, new java.lang.ref.SoftReference(px));
		return px;
	    } else {
		return pc;
	    }
	}
    }

    /**
     * This method maps a proxy object of a Constructor to a PnutsFunction.
     * Call of the resulting function is faster than reflection API calls.
     *
     * @param cons	a constructor
     * @return	an instance the constructor creates.
     */
    public static PnutsFunction makeProxy(Constructor cons) {
	final DynamicProxy px =
	    DynamicProxyFactory.makeProxy(cons,
					  Compiler.createCodeLoader(getClassLoader(cons.getDeclaringClass()), true));

	if (cons.getParameterTypes().length == 0){
	    return new PnutsFunction(){
		public Object exec(Object[] args, Context context){
		    return px.invoke(null);
		}
	    };
	} else {
	    return new PnutsFunction(){
		public Object exec(Object[] args, Context context){
		    return px.invoke(null, args);
		}
	    };
	}
    }

    /**
     * This method maps a proxy object of a Method to a PnutsFunction.
     * Call of the resulting function is expected to be faster than
     * reflection API calls.
     *
     * @param method	a method
     * @return	the result of the 'method' call.
     */
    public static PnutsFunction makeProxy(Method method) {
	final DynamicProxy px =
	    DynamicProxyFactory.makeProxy(method,
					  Compiler.createCodeLoader(getClassLoader(method.getDeclaringClass()), true));
	final String name = method.getName();
	boolean _static  = Modifier.isStatic(method.getModifiers());
	int nargs = method.getParameterTypes().length;

	if (_static){
	    if (nargs == 0){
		return new PnutsFunction(){
		    public String getName(){
			return name;
		    }
		    public Object exec(Object[] args, Context context){
			return px.invoke(null);
		    }
		};
	    } else {
		return new PnutsFunction(){
		    public String getName(){
			return name;
		    }
		    public Object exec(Object[] args, Context context){
			return px.invoke(null, args);
		    }
		};
	    }
	} else {
	    if (nargs == 0){
		return new PnutsFunction(){
		    public String getName(){
			return name;
		    }
		    public Object exec(Object[] args, Context context){
			return px.invoke(args[0]);
		    }
		};
	    } else {
		return new PnutsFunction(){
		    public String getName(){
			return name;
		    }
		    public Object exec(Object[] args, Context context){
			Object target = args[0];
			System.arraycopy(args, 1, args, 0, args.length - 1);
			return px.invoke(target, args);
		    }
		};
	    }
	}
    }

    static ClassLoader getClassLoader(final Class clazz){
	if (Pnuts.isJava2()){
	    return (ClassLoader)AccessController.doPrivileged(new PrivilegedAction(){
		    public Object run(){
			return clazz.getClassLoader();
		    }
		});
	} else {
	    return clazz.getClassLoader();
	}
    }

    static class NoMemberFoundException extends RuntimeException { }

    static MethodFinder methodFinder;

    static {
	if (Pnuts.isJava2()){
	    methodFinder = new Java2MethodFinder();
	    if (DEBUG){
		System.out.println("Java2MethodFinder is used");
	    }
	} else {
	    methodFinder = new ClassicMethodFinder();
	    if (DEBUG){
		System.out.println("ClassicMethodFinder is used");
	    }
	}
    }
}
