/*
 * @(#) FileWriterHandler.java	1.2 00/07/06
 *
 * Copyright (c) 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.compiler;

import pnuts.lang.PnutsException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * This class is a concrete class of ClassFileHandler.
 * When this is passed to Compiler.compile(..., ClassFileHandler),
 * compiled class files are saved in the directory specified with the constructor.
 */
public class FileWriterHandler implements ClassFileHandler {
    private File dir;
    private boolean verbose;

    public FileWriterHandler(File dir){
	this.dir = dir;
    }

    public void setVerbose(boolean flag){
	this.verbose = flag;
    }

    public void handle(CompileContext cc, ClassFile cf) {
	try {
	    write(cf, dir);
	} catch (IOException e){
	    throw new PnutsException(e);
	}
    }

    void write(ClassFile cf, File base) throws IOException {
	String className = cf.getClassName();
	int idx = className.lastIndexOf('.');
	File file = null;
	if (idx > 0){
	    String dirname = className.substring(0, idx).replace('.', File.separatorChar);
	    File dir = new File(base, dirname);
	    dir.mkdirs();
	    file = new File(dir, className.substring(idx + 1) + ".class");
	} else {
	    file = new File(base, className + ".class");
	}
	FileOutputStream fin = new FileOutputStream(file);
	DataOutputStream dout = new DataOutputStream(fin);
	cf.write(dout);
	dout.close();
	if (verbose){
	    System.out.println(file);
	}
    }

    public Object getResult(CompileContext cc){
	return null;
    }
}
