/*
 * @(#) Frame.java	1.1 99/10/25
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.compiler;

import java.util.Vector;
import java.util.Hashtable;

/**
 * This class represents a function scope.
 */
class Frame {
    Frame parent;
    SymbolSet symbolSet;
    SymbolSet bottom;
    String fname;
    String[] locals;
    Vector imports = new Vector();
    Hashtable exports = new Hashtable();

    Frame(){
	openLocal();
    }

    Frame(String locals[], String fname, Frame parent){
	this.parent = parent;
	if (fname != null){
	    this.fname = fname.intern();
	}
	for (int i = 0; i < locals.length; i++){
	    locals[i] = locals[i].intern();
	}
	this.locals = locals;
	this.bottom = new SymbolSet(null);
	this.symbolSet = bottom;
	openLocal();
    }

    void openLocal(){
	this.symbolSet = new SymbolSet(symbolSet);
    }

    void closeLocal(){
	this.symbolSet = symbolSet.parent;
    }

    boolean setReference(String sym){
	LocalInfo info = symbolSet.assoc(sym);
	if (info != null){
	    info.initialized = true;
	    return true;
	} else {
	    return false;
	}
    }

    Reference getReference(String sym){
	LocalInfo info = symbolSet.assoc(sym);

	if (info != null){
	    return new Reference(sym, info.map, info.index, info.initialized);
	}
	if (locals != null){
	    for (int i = 0; i < locals.length; i++){
		if (locals[i] == sym){
		    return new Reference(sym, 1, i, true);
		}
	    }
	}
	return null; // not found
    }

    public void declare(String symbol, int key, int idx){
	bottom.add(symbol, key, idx);
    }

    public void declare(String symbol, int key){
	bottom.add(symbol, key);
    }

    public void _declare(String symbol, int key, int idx){
	symbolSet.add(symbol, key, idx);
    }

    public void _declare(String symbol, int key){
	symbolSet.add(symbol, key);
    }
}
