/*
 * @(#) SymbolSet.java	1.2 99/12/20
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.compiler;

/**
 * This class represents a variable scope made by functions,
 * for/foreach statements, and so on.
 *
 * @see pnuts.compiler.Frame
 * @see pnuts.compiler.LocalInfo
 */
final class SymbolSet {
    SymbolSet parent;
    String keys[] = new String[4];
    LocalInfo info[] = new LocalInfo[4];
    int count = 0;

    SymbolSet(){
    }

    SymbolSet(SymbolSet parent){
	this.parent = parent;
	if (parent != null){
	    this.count = parent.count;
	}
    }

    void add(String sym, int map){
	add(sym, map, 0);
    }

    void add(String sym, int map, int idx){
	if (count > keys.length - 1){
	    String newKeys[] = new String[keys.length + count];
	    System.arraycopy(keys, 0, newKeys, 0, keys.length);
	    keys = newKeys;
	    LocalInfo newInfo[] = new LocalInfo[info.length + count];
	    System.arraycopy(info, 0, newInfo, 0, info.length);
	    info = newInfo;
	}
	keys[count] = sym.intern();
	info[count] = new LocalInfo(map, idx, false);
	count++;
    }

    LocalInfo assoc(String key){
	return _assoc(key.intern());
    }

    LocalInfo _assoc(String key){
	for (int i = 0; i < count; i++){
	    if (keys[i] == key){
		return info[i];
	    }
	}
	if (parent != null){
	    return parent._assoc(key);
	} else {
	    return null;
	}
    }
}
