/*
 * @(#) ZipWriterHandler.java	1.1 99/12/20
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.compiler;

import pnuts.lang.PnutsException;
import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;

/**
 * This class is a concrete class of ClassFileHandler.
 * When this is passed to Compiler.compile(..., ClassFileHandler),
 * compiled class files are added to the ZipOutputStream specified with the constructor.
 */
public class ZipWriterHandler implements ClassFileHandler {
    private ZipOutputStream zout;
    private ByteArrayOutputStream bout;
    private DataOutputStream dout;
    private boolean verbose;

    public ZipWriterHandler(ZipOutputStream zout){
	this.zout = zout;
	bout = new ByteArrayOutputStream();
	dout = new DataOutputStream(bout);
    }

    public void setVerbose(boolean flag){
	this.verbose = flag;
    }

    public void handle(CompileContext cc, ClassFile cf) {
	try {
	    bout.reset();
	    cf.write(dout);
	    ZipEntry entry = new ZipEntry(cf.getClassName().replace('.', '/') + ".class");
	    zout.putNextEntry(entry);
	    bout.writeTo(zout);
	    if (verbose){
		System.out.println(entry);
	    }
	} catch (IOException e){
	    throw new PnutsException(e);
	}
    }

    public Object getResult(CompileContext cc){
	return null;
    }
}
