/*
 * @(#) JarLoader.java	1.1 99/11/01
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext;

import java.net.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;

class JarLoader implements Loader {
    private ZipFile jar;
    private URL csu;
    private URL base;

    JarLoader(URL url) throws IOException {
	base = new URL("jar", "", -1, url + "!/");
	jar = getJarFile(url);
	csu = url;
    }

    private ZipFile getJarFile(URL url) throws IOException {
	if ("file".equals(url.getProtocol())) {
	    String path = url.getFile().replace('/', File.separatorChar);
	    File file = new File(path);
	    if (!file.exists()) {
		throw new FileNotFoundException(path);
	    }
	    return new ZipFile(path);
	}
	throw new RuntimeException("not supported");
    }

    public Resource getResource(final String name) {
	final ZipEntry entry = jar.getEntry(name);
	if (entry != null) {
	    final URL url;
	    try {
		url = new URL(base, name);
	    } catch (MalformedURLException e) {
		return null;
	    } catch (IOException e) {
		return null;
	    }
	    return new Resource() {
		public URL getURL(){
		    return url;
		}
		public InputStream getInputStream() throws IOException {
		    return jar.getInputStream(entry);
		}
		public int getContentLength(){
		    return (int)entry.getSize();
		}
	    };
	}
	return null;
    }
}

