/*
 * @(#) PropertyAccessor.java	1.2 99/12/22
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext;

import pnuts.lang.*;
import pnuts.lang.Runtime;
import pnuts.lang.Property;

/**
 * When a pnuts.lang.Property object has functions as its values,
 * the functions can be called as follows.
 * <pre>
 * object.name = function () null
 * (object.name)()
 * </pre>
 * <p>
 * This class makes this expression a little simpler.
 * </p>
 * <pre>
 * exported = PropertyAccessor(object)
 * exported.name()
 * </pre>
 * <p>
 * A concrete class of pnuts.lang.AbstractData can be used for the same purpose,
 * but the name of properties and its public methods may conflict.
 * </p>
 */
public class PropertyAccessor implements AbstractData {
    private Property property;

    public PropertyAccessor(Property property){
	this.property = property;
    }

    public Object get(String name, Context context){
	return property.get(name, context);
    }

    public void set(String name, Object object, Context context){
	property.set(name, object, context);
    }

    public Object invoke(String name, Object args[], Context context){
	Object o = property.get(name, context);
	if (o instanceof PnutsFunction){
	    return ((PnutsFunction)o).call(args, context);
	} else if (o instanceof Class){
	    return Runtime.callConstructor(context, (Class)o, args, null);
	} else {
	    throw new PnutsException("funcOrType.expected", new Object[]{o}, context);
	}
    }

    public Property getProperty(){
	return property;
    }

    public static Property getProperty(PropertyAccessor accessor){
	return accessor.getProperty();
    }

    public String toString(){
	return getClass().getName() + "{" + property + "}";
    }
}
