/*
 * @(#) Builtin.java	1.13 00/08/08
 *
 * Copyright (c) 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.io.InputStream;
import java.io.Reader;
import java.io.FileNotFoundException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.net.URL;

/**
 * Builtin functions
 */
final class Builtin extends PnutsFunction {

    protected Builtin(String name){
	super(name);
    }

    protected Object exec(Object[] args, Context context){
	if (name == _getContext){
	    if (args.length == 0){
		return context;
	    }
	} else if (name == _package){
	    if (args.length == 1){
		Package pkg;
		if (args[0] instanceof Package){
		    pkg = (Package)args[0];
		} else if (args[0] != null){
		    pkg = Package.getPackage((String)args[0]);
		} else {
		    pkg = Package.getInstance(null, Package.globalPackage);
		}
		pkg.init(context);
		return context.currentPackage;
	    } else if (args.length == 0){
		return context.currentPackage;
	    }
	} else if (name == _import){
	    if (args.length == 1){
		if (args[0] == null){
		    context.importEnv = new ImportEnv();
		    return null;
		} else {
		    String s = (String)args[0];
		    if (!s.endsWith("*")){
			context.addClassToImport(s);
		    } else {
			int idx = s.lastIndexOf('.');
			if (idx > 0){
			    context.addPackageToImport(s.substring(0, idx));
			} else {
			    context.addPackageToImport("");
			}
		    }
		    return null;
		}
	    } else if (args.length == 0){
		return context.importEnv.list();
	    }
	} else if (name == _throw){
	    if (args.length == 1){
		if (args[0] instanceof Throwable){
		    throw new PnutsException((Throwable)args[0], context);
		} else {
		    throw new PnutsException(args[0].toString(), context);
		}
	    }
	} else if (name == _defined){
	    if (args.length == 1){
		return new Boolean(context.defined((String)args[0]));
	    }
	} else if (name == _quit){
	    if (args.length == 0){
		throw new Escape();
	    } else if (args.length == 1){
		throw new Escape(args[0]);
	    }
	} else if (name == _eval){
	    Context c = null;
	    if (args.length == 1){
		c = (Context)((Context)context).clone(false);
		c.parent = context;
		c.eval = true;
		return Pnuts.eval((String)args[0], c);
	    } else if (args.length == 2){
		if (args[1] instanceof Context){
		    c = (Context)((Context)args[1]).clone(false);
		} else if (args[1] instanceof Package){
		    c = (Context)context.clone(false);
		    c.currentPackage = (Package)args[1];
		} else {
		    c = (Context)context.clone(false);
		    c.currentPackage = Package.getPackage((String)args[1]);
		}
		c.parent = context;
		c.eval = true;
		return Pnuts.eval((String)args[0], c);
	    }
	} else if (name == _loadFile){
	    try {
		Context c = null;
		if (args.length > 0){
		    if (args.length > 1){
			c = (Context)((Context)args[1]).clone(false);
		    } else {
			c = (Context)context.clone();
		    }
		    return Pnuts.loadFile((String)args[0], c);
		}
	    } catch (FileNotFoundException e){
		throw new PnutsException(e, context);
	    }
	} else if (name == _load){
	    try {
		Context c = null;
		if (args.length > 0){
		    if (args.length > 1){
			c = (Context)args[1];
			c = (Context)c.clone(false);
		    } else {
			c = (Context)context.clone();
		    }
		    if (args[0] instanceof InputStream){
			return Pnuts.load((InputStream)args[0], c);
		    } else if (args[0] instanceof Reader){
			return Pnuts.load((Reader)args[0], c);
		    } else if (args[0] instanceof URL){
			return Pnuts.load((URL)args[0], c);
		    } else {
			return Pnuts.load((String)args[0], c);
		    }
		}
	    } catch (FileNotFoundException e){
		throw new PnutsException(e, context);
	    }
	} else if (name == _require){
	    if (args.length == 1){
		try {
		    Context c = (Context)context.clone();
		    Pnuts.require((String)args[0], c);
		    return null;
		} catch (FileNotFoundException e){
		    throw new PnutsException(e, context);
		}
	    }
	} else if (name == _class){
	    if (args.length == 1){
		try {
		    return Pnuts.loadClass((String)args[0], context);
		} catch (ClassNotFoundException e){
		    return null;
		}
	    }
	} else if (name == _use){
	    if (args.length == 1){
		context.usePackage((String)args[0]);
		return context.usedPackages();
	    }
	}
	throw new PnutsException("function.notDefined",
				 new Object[]{name, new Integer(args.length)},
				 context);
    }

    public String toString(){
	return "<builtin " + name + ">";
    }

    public String unparse(int narg){
	return null;
    }

    public String[] getImportEnv(int narg){
	return null;
    }

    public Package getPackage(){
	return Package.globalPackage;
    }

    public boolean isBuiltin(){
	return true;
    }
}
