/*
 * @(#) ImportEnv.java	1.17 00/07/13
 *
 * Copyright (c) 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

class ImportEnv implements Serializable, Cloneable {

    static final long serialVersionUID = -4937065964296201727L;

    /**
     * @serial
     */
    Hashtable imports = new Hashtable(16);

    /**
     * @serial
     */
    Hashtable importedClasses = new Hashtable(16);

    /**
     * @serial
     */
    Vector pkgOrder = new Vector();

    /**
     * @serial
     */
    Vector classOrder = new Vector();

    transient Cache cache = new Cache(32, false);
    transient Cache failCache = new Cache(32, false);

    Import getImport(String name){
	Import imp = (Import)imports.get(name);
	if (imp != null){
	    return imp;
	} else {
	    return new Import(name);
	}
    }

    synchronized void addClass(String className){
	int idx = className.lastIndexOf('.');
	String name = className.substring(idx + 1);
	Vector vec = (Vector)importedClasses.get(name);
	if (vec == null){
	    importedClasses.put(name, vec = new Vector(4));
	}
	if (classOrder.contains(className)){
	    classOrder.removeElement(className);
	}
	classOrder.addElement(className);

	if (vec.contains(className)){
	    vec.removeElement(className);
	}
	vec.addElement(className);
    }

    synchronized void addPackage(String pkgname){
	Import imp = (Import)imports.get(pkgname);
	if (imp != null){
	    pkgOrder.removeElement(imp);
	} else {
	    imp = getImport(pkgname);
	}
	pkgOrder.addElement(imp);
	imports.put(pkgname, imp);
	cache = new Cache(32, false);
	failCache = new Cache(32, false);
    }
 
    public Class get(String sym, Context context){
	Object v = cache.get(sym);
	if (v != null){
	    return (Class)v;
	}

	Vector vec = (Vector)importedClasses.get(sym);
	if (vec != null){
	    int size = vec.size();
	    for (int i = size - 1; i >= 0; i--){
		try {
		    return Pnuts.loadClass((String)vec.elementAt(i), context);
		} catch (ClassNotFoundException cnf){}
	    }
	}
	if (Package.globalPackage.autoloadTable.get(sym) != null){
	    return null;
	}
	int symlen = sym.length();
	StringBuffer buf = new StringBuffer(sym);
	boolean lowercase = !Character.isUpperCase(sym.charAt(0));

 	int size = pkgOrder.size();
	for (int i = size - 1; i >= 0; i--){
	    Import imp = (Import)pkgOrder.elementAt(i);
	    buf.setLength(symlen);
	    String name = imp.getName();
	    if (lowercase && name.startsWith("java.")){
		continue;
	    }
	    buf.append(name);
	    String key = buf.toString();
	    if (failCache.get(key) != null){
		continue;
	    }
	    Class value = imp.get(sym, context);
	    if (value != null){
		cache.put(sym, value);
		return value;
	    } else {
		failCache.put(key, key);
	    }
	}
	return null;
    }

    String[] list(){
	String ret[] = new String[pkgOrder.size() + classOrder.size()];
	int i = ret.length - 1;
	for (Enumeration e = pkgOrder.elements(); e.hasMoreElements(); ){
	    String name = ((Import)e.nextElement()).name;
	    if ("".equals(name)){
		ret[i--] = "*";
	    } else {
		ret[i--] = name + ".*";
	    }
	}
	for (Enumeration e = classOrder.elements(); e.hasMoreElements(); ){
	    ret[i--] = (String)e.nextElement();
	}
	return ret;
    }

    void reset(){
	cache = new Cache(32, false);
	failCache = new Cache(32, false);
	for (Enumeration e = pkgOrder.elements(); e.hasMoreElements(); ){
	    ((Import)e.nextElement()).reset();
	}
    }

    public Object clone(){
	try {
	    ImportEnv c = (ImportEnv)super.clone();
	    c.imports = (Hashtable)imports.clone();
	    c.importedClasses = (Hashtable)importedClasses.clone();
	    c.pkgOrder = (Vector)pkgOrder.clone();
	    c.classOrder = (Vector)classOrder.clone();
	    c.cache = new Cache(32, false);
	    c.failCache = new Cache(32, false);
	    return c;
	} catch (Throwable t){
	    throw new InternalError();
	}
    }

    private synchronized void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException {
	s.defaultReadObject();
	cache = new Cache(32, false);
	failCache = new Cache(32, false);
    }
}
