/*
 * @(#) Java2Dependent.java	1.6 00/08/01
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.security.*;
import java.io.File;
import java.io.FilePermission;
import java.io.InputStream;
import java.io.IOException;
import java.util.Iterator;
import java.net.URL;
import java.net.SocketPermission;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

/**
 * This class defines a set of version dependent methods for Java 2.
 */
class Java2Dependent extends VersionDependent {
    private final static boolean DEBUG = false;

    URL getResource(final String s){
	return (URL)AccessController.doPrivileged(new PrivilegedAction(){
	    public Object run(){
		ClassLoader cl = null;
		String res = s;
		try {
		    cl = ClassLoader.getSystemClassLoader();
		} catch (SecurityException t){ } // skip
		if (cl != null){
		    if (s.startsWith("/")){
			res = s.substring(1);
		    }
		    return cl.getResource(res);
		}
		return Pnuts.class.getResource(res);
	    }
	});
    }

    Method[] getMethods(final Class c){
	return (Method[])AccessController.doPrivileged(new PrivilegedAction(){
	    public Object run(){
		return c.getMethods();
	    }
	});
    }

    int compare(Object o1, Object o2){
	if (o1 instanceof Comparable && o2 instanceof Comparable){
	    int c = ((Comparable)o1).compareTo(o2);
	    if (c < 0){
		return -1;
	    } else if (c > 0){
		return 1;
	    } else {
		return 0;
	    }
	} else {
	    return super.compare(o1, o2);
	}
    }

    Object doIterator(Object target,
		      SimpleNode node,
		      SimpleNode block,
		      PnutsInterpreter interp,
		      Context context)
    {
	if (!(target instanceof Iterator)){
	    super.doIterator(target, node, block, interp, context);
	}
	Object last = null;
	Iterator it = (Iterator)target;
	context.openLocal(new String[]{node.str});
	Object[] attr = (Object[])node.info;
	while (true){
	    try {
		if (!it.hasNext()){
		    break;
		}
		context.bind(node.str, it.next());
		last = block.accept(interp, context);
		if (attr[PnutsInterpreter.BROKEN] == Boolean.TRUE){
		    attr[PnutsInterpreter.BROKEN] = Boolean.FALSE;
		    last = attr[PnutsInterpreter.PASSED];
		    break;
		}
	    } catch (PnutsException e0){
		throw e0;
	    } catch (Throwable e){
		throw new PnutsException(e, context);
	    }
	}
	context.closeLocal();
	return last;
    }

    Object reInvoke(IllegalAccessException t, final Method method, Object target, Object[] args)
	throws IllegalAccessException, IllegalArgumentException, InvocationTargetException
    {
	AccessController.doPrivileged(new PrivilegedAction(){
	    public Object run(){
		method.setAccessible(true);
		return null;
	    }
	});
	return method.invoke(target, args);
    }

    void replace(StringBuffer buf, int from, int to, String str){
	buf.replace(from, to, str);
    }

    void checkPackageAccess(Package pkg, String action){
	SecurityManager security = System.getSecurityManager();
	if (security == null) {
	    return;
	}
	String name = pkg.getName();
	if (name == null){
	    return;
	}
	if (name == ""){
	    name = "::";
	}
	AccessController.checkPermission(new PackagePermission(name, action));
    }

    Object execute(final Runtime runtime, final Context context){
	return AccessController.doPrivileged(new PrivilegedAction() {
	    public Object run() {
		return runtime.execute(context);
	    }
	});
    }
}
