/*
 * @(#) StackFrame.java	1.2 99/11/01
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

class StackFrame {
    StackFrame parent;
    SymbolTable symbolTable;

    StackFrame(){
	this.symbolTable = new SymbolTable();
    }

    StackFrame(String locals[], StackFrame parent){
	this.parent = parent;
	this.symbolTable = new SymbolTable();
	openLocal(locals);
    }

    public void openLocal(String locals[]){
	symbolTable = new SymbolTable(symbolTable);

	for (int i = 0; i < locals.length; i++){
	    symbolTable.put0(locals[i], null);
	}
    }

    public void closeLocal(){
	symbolTable = symbolTable.parent;
    }

    public Binding lookup(String sym){
	return symbolTable.lookup(sym);
    }

    public Object get(String sym){
	return symbolTable.get(sym);
    }

    public void put(String sym, Object value){
	symbolTable.put(sym, value);
    }


    public void put1(String sym, Object value){
	SymbolTable t = symbolTable;
	while (t.parent != null){
	    t = t.parent;
	}
	t.put0(sym, value);
    }

    public final void bind(String sym, Object value){
	symbolTable.put0(sym, value);
    }

    void makeLexicalScope(SymbolTable out){
	makeLexicalScope(symbolTable, out);
    }

    void makeLexicalScope(SymbolTable env, SymbolTable out){
	if (env.parent != null){
	    makeLexicalScope(env.parent, out);
	}
	for (int i = 0, j = 0; i < env.count; j++){
	    Binding b = env.table[j];
	    while (b != null){
		out.put(b.name, b);
		i++;
		b = b.chain;
	    }
	}
    }

    /*
    void dump(){
	symbolTable.dump();
	if (parent != null){
	    parent.dump();
	}
    }
    */
}
