/*
 * @(#) VersionDependent.java	1.4 00/02/17
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.io.InputStream;
import java.net.URL;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.InvocationTargetException;
import pnuts.lang.Runtime;

/**
 * This class declares a set of methods that depend on Java version,
 * and defines the default behavior (JDK1.1).
 */
class VersionDependent {

    static VersionDependent getInstance(){
	if (Pnuts.isJava2()){
	    return new Java2Dependent();
	} else {
	    return new VersionDependent();
	}
    }

    URL getResource(String s){
	return Pnuts.class.getResource(s);
    }

    Method[] getMethods(Class c){
	return c.getMethods();
    }

    int compare(Object o1, Object o2){
	return o1.equals(o2) ? Arithmetic.EQUAL : Arithmetic.NOT_EQUAL;
    }

    Object doIterator(Object target,
		      SimpleNode node,
		      SimpleNode block,
		      PnutsInterpreter interp,
		      Context context)
    {
	throw new PnutsException("illegal.type.foreach", PnutsException.NO_PARAM, context);
    }

    Object reInvoke(IllegalAccessException t, Method method, Object target, Object[] args)
	throws IllegalAccessException, IllegalArgumentException, InvocationTargetException
    {
      if (target != null){
	Method m = findCallableMethod(target.getClass(),
				      method.getName(),
				      method.getParameterTypes());
	if (m != null){
	    return m.invoke(target, args);
	}
      }
      throw t;
    }

    private static Method findCallableMethod(Class clazz, String name, Class args[]){
      Method method = null;
      if (!clazz.isInterface()){
	Class it[] = clazz.getInterfaces();
	for (int i = 0; i < it.length; i++){
	    if (Modifier.isPublic(it[i].getModifiers())){
		try {
		    method = it[i].getMethod(name, args);
		    if (method != null){
			return method;
		    }
		} catch (NoSuchMethodException nsme){
		}
	    }
	}
	try {
	    method = clazz.getSuperclass().getMethod(name, args);
	    if (method != null){
		if (Modifier.isPublic(method.getDeclaringClass().getModifiers())){
		    return method;
		}
	    }
	} catch (NoSuchMethodException nsme){
	}
      }
      return null;
    }

    void replace(StringBuffer buf, int start, int end, String str){
	int count = buf.length();
	int chars = count - end;
	if (start >= 0 && start < count){
	    if (chars > 0){
		char[] tmp = new char[chars];
		buf.getChars(end, count, tmp, 0);
		buf.setLength(start);
		buf.append(str);
		buf.append(tmp);
	    } else {
		buf.setLength(start);
		buf.append(str);
	    }
	}
    }

    void checkPackageAccess(Package pkg, String action){
	/* do nothing */
    }

    Object execute(Runtime runtime, Context context){
	return runtime.execute(context);
    }
}
