/*
 * @(#) PnutsServlet.java	1.5 00/08/07
 *
 * Copyright (c) 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.servlet;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.MalformedURLException;
import pnuts.lang.Pnuts;
import pnuts.lang.PnutsImpl;
import pnuts.lang.Context;
import pnuts.lang.Package;
import pnuts.lang.Jump;
import pnuts.ext.CachedPnutsImpl;

/**
 * Servlet for Pnuts scripting
 */
public class PnutsServlet extends HttpServlet {

    private static final boolean DEBUG = false;
    private PnutsImpl pnutsImpl;
    private Context context;

    public PnutsServlet(){
    }

    public void init() throws ServletException {
	ServletConfig conf = getServletConfig();
	context = new Context();
	String module = conf.getInitParameter("module");
	if (module != null){
	    context.usePackage(module);
	}
	String name = conf.getInitParameter("initialScript");
	try {
	    if (name != null){
		Pnuts.require(name, context);
	    }
	} catch (FileNotFoundException e){
	}
	String compile = conf.getInitParameter("compile");
	pnutsImpl = new CachedPnutsImpl(Boolean.valueOf(compile).booleanValue());
    }

    public void service(HttpServletRequest request,
			HttpServletResponse response)
	throws ServletException, IOException
    {
	Package pkg = new Package();
	Context c = (Context)context.clone();
	c.setCurrentPackage(pkg);
	try {
	    c.setOutputStream(response.getOutputStream());
	    c.setPnutsImpl(pnutsImpl);

	    pkg.set("response", response, c);
	    pkg.set("request", request, c);

	    String path = getServletContext().getRealPath(request.getServletPath());
	    File file = new File(path);
	    try {
		Pnuts.setClassLoader(new URLClassLoader(new URL[]{file.getParentFile().toURL()}),
				     c);
	    } catch (MalformedURLException mue){
	    }
	    Pnuts.loadFile(path, c);

	} catch (Jump jump){
	    /* returned from top-level */
	} catch (Exception e){
	    PrintWriter errorStream = c.getErrorStream();
	    if (errorStream != null){
		if (Pnuts.isVerbose()){
		    e.printStackTrace(errorStream);
		    errorStream.flush();
		} else {
		    errorStream.println(e);
		    errorStream.flush();
		}
	    }
	    throw new ServletException("Script Error: " + e);
	}
    }
}
