/*
 * @(#) BytecodeClassLoader.java	1.1 00/08/01
 *
 * Copyright (c) 1997,2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.util;

import java.io.InputStream;
import java.io.IOException;
import pnuts.lang.PnutsException;

/**
 *
 * @version	1.1
 */
public class BytecodeClassLoader extends ClassLoader {

    private boolean canDefine = true;

    /**
     * load <em>name</em> class.
     * If <em>resolve</em> is true resolveClass() is called.
     */
    protected Class loadClass(String name, boolean resolve) throws ClassNotFoundException {
	Class cl = findLoadedClass(name);
	if (cl == null) {
	    try {
		return findSystemClass(name);
	    } catch (Throwable e) {  } // ignore
	    cl = findClass(name);
	}
	if (cl == null) {
	    throw new ClassNotFoundException(name);
	}
	if (resolve) {
	    resolveClass(cl);
	}
	return cl;
    }

    public void resolve(Class c) { 
	resolveClass(c);
    }

    private void check(){
	if (!canDefine){
	    throw new RuntimeException("Can't define class");
	}
    }

    public void seal(){
	canDefine = false;
    }

    /**
     * Read Bytecode <em>data</em> <em>length</em> bytes from <em>offset</em>
     * and define <em>name</em> class.
     */
    public Class define(String name, byte data[], int offset, int length){
	check();
	Class c = super.defineClass(name, data, offset, length);
	return c;
    }
}
