/*
 * @(#) psh.c	1.2 00/04/12
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <windows.h>
#include <shlobj.h>
#include <objidl.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <io.h>
#include <winsock.h>

/*
 * This program is the client side of the pserv/psh environment.
 * Psh reads Pnuts scripts from files or command-line expression and
 * sends them to the pserv program through a socket connection.
 * Pserv receives scripts sent by psh and execute them. 
 *
 * Lines that starts with '#' is ignored. And a variable $args is predefined
 * as an array of command arguments.
 *
 * Usage:
 *   psh [ -help | [ -port:NNNN ] [ -e expr | script_file, arg1, ...]]
 *
 */

#define INADDR_NONE 0xffffffff

#define HOSTNAME "127.0.0.1"


/*
 * Connect to host:port
 */
int do_connect(char* host, int port)
{
  int fd;
  unsigned long inaddr;
  struct hostent *hp;
  struct sockaddr_in srv_addr;
  
  memset((char*)&srv_addr, 0, sizeof(srv_addr));
  srv_addr.sin_family = AF_INET;

  srv_addr.sin_port = htons((unsigned short)port);

  if ((inaddr = inet_addr(host)) != INADDR_NONE){
    memcpy((char*)&srv_addr.sin_addr, (char*)&inaddr, sizeof(inaddr));
  } else {
    if ((hp = gethostbyname(host)) == NULL){
      return -1;
    }
    memcpy((char*)&srv_addr.sin_addr, hp->h_addr, hp->h_length);
  }

  if (port < 0){
    return -1;
  }
  if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0){
	return -1;
  }
  if (connect(fd, (struct sockaddr*)&srv_addr, sizeof(srv_addr)) < 0){
	  close(fd);
    return -1;
  }
  return fd;
}

/*
 * Get the home directory
 */
char* get_home()
{
  LPITEMIDLIST itemList;

  if (SUCCEEDED(SHGetSpecialFolderLocation(NULL,
					   CSIDL_DESKTOPDIRECTORY,
					   &itemList))) {
    char szUserPath[MAX_PATH];
    char *p;
    SHGetPathFromIDList(itemList, szUserPath);

    /* Get the parent of Desktop directory */
    p = strrchr(szUserPath, '\\');
    if (p) {
      *p = '\0';
    }
    return strdup(szUserPath);
  } else {
    return "C:\\";
  }
}

/*
 * Check if the file exists
 */
int file_exists(char* file){
  struct stat st;
  if (stat(file, &st) != 0){
    return 0;
  }
  return (st.st_mode & S_IFMT) == S_IFREG;
}

/*
 * Read the cookie file into buf.
 */
int read_cookie(char *buf)
{
  char* home;
  char* path;

  home = get_home();


  path = (char*)malloc(strlen(home) + 15);
  if (path == NULL){
    return -1;
  }
  sprintf(path, "%s/.pnuts/cookie", home);
  if (file_exists(path)){
    int fd = open(path, O_RDONLY);
    if (read(fd, buf, 10) != 10){
      return -1;
    }
    return 0;
  }
  return -1;
}

int do_write(int fd, char* pt, int len)
{

	int n;

	if ((n = send(fd, pt, len, 0)) < 0){
		MessageBox(NULL, "send error", "error", MB_OK);
	}

	return n;
}


int do_read(int fd, char *pt, int len){

	int n = recv(fd, pt, len, 0);

	return n;

}


void read_stdin(int sock)
{
  int maxfdp1, nfound, nread;
  char buf[512];
  fd_set readmask;


  FD_ZERO(&readmask);
  for (;;){
    FD_SET(0, &readmask);
    FD_SET(sock, &readmask);
    maxfdp1 = sock + 1;

    nfound = select(maxfdp1, &readmask, (fd_set*)0, (fd_set*)0, (struct timeval*)0);
    if (nfound < 0){
		MessageBox(NULL, "select error", "error", MB_OK);
      exit(-1);
    }
    if (FD_ISSET(0, &readmask)){
      nread = read(0, buf, sizeof(buf));
      if (nread < 0){
			MessageBox(NULL, "read error", "error", MB_OK);
	exit(-1);
      } else if (nread == 0){
	break;
      }

	  if (do_write(sock, buf, nread) != nread){
		  	MessageBox(NULL, "write error", "error", MB_OK);
	exit(-1);
      }
    }
    if (FD_ISSET(sock, &readmask)){
      nread = do_read(sock, buf, sizeof(buf));
      if (nread <= 0){
	break;
      }
      if (write(1, buf, nread) != nread){
			MessageBox(NULL, "write error", "error", MB_OK);
	exit(-1);
      }
    }
  }
}

/*
 * Read all data from fp and write to sock.
 * The first line which starts with '#!' is discarded.
 */
void read_input(FILE* fp, int sock)
{
  char line[512];

  int len;



  if (fgets(line, sizeof(line), fp) != NULL){

    if (line[0] != '#' || line[1] != '!'){

      len = strlen(line);

      line[len] = '\n';

      do_write(sock, line, len + 1);
    }
  }
  while (fgets(line, sizeof(line), fp) != NULL){
    len = strlen(line);
    line[len] = '\n';
    do_write(sock, line, len + 1);
  }
}

/*
 * Receive output from server
 */
void receive_output(int sock)
{
  int i, n;
  char line[512];

  while ((n = do_read(sock, line, sizeof(line))) > 0){

	  for (i = 0; i < n; i++){
		  if (line[i] != 0x0d){
			putchar(line[i]);
		  }
	  }
  /*  write(0, line, n);*/
  }
}

char* append(char* buf, int* size, int* capacity, char* str)
{
  char* ret = buf;
  int len = strlen(str);
  if (*size + len + 1 > *capacity){
    ret = (char*)malloc(*capacity = (*capacity + *size + len + 1));
    if (ret == NULL){
      return NULL;
    }
    memcpy(ret, buf, *size);
  }
  memcpy(ret + *size, str, len);
  ret[*size + len] = '\0';
  *size += len;
  return ret;
}

char* escape(char* str)
{
  int i = 0;
  char* s = str;

  while (*s){
    switch (*s){
    case '\\':
    case '"':
      i++;
      break;
    }
    s++;
  }
  if (i > 0){
    char *ret;
    char *ss = ret = (char*)malloc(strlen(str) + i + 1);
    if (ss == NULL){
      return NULL;
    }
    s = str;
    while (*s){
      switch (*s){
      case '\\':
      case '"':
	*ss++ = '\\';
	break;
      }
      *ss++ = *s++;
    }
    *ss = *s;
    return ret;
  } else {
    return str;
  }
}

/*
 * Define $args=[argv[0],argv[1],..]
 */
int set_args(int sock, int argc, char** argv)
{
  int i;
  int count = 0;
  int cap = 64;
  char *buf = (char*)malloc(cap);

  buf = append(buf, &count, &cap, "$args=[\"");
  buf = append(buf, &count, &cap, escape(argv[1]));
  buf = append(buf, &count, &cap, "\"");
  for (i = 2; i < argc; i++){
    buf = append(buf, &count, &cap, ",\"");
    buf = append(buf, &count, &cap, escape(argv[i]));
    buf = append(buf, &count, &cap, "\"");
  }
  buf = append(buf, &count, &cap, "]\n");

  do_write(sock, buf, strlen(buf));

  return 0;
}


void init_winsock()

{

	WSADATA wsadata;

	HANDLE hWinsock;



	hWinsock = LoadLibrary("ws2_32.dll");

	if (hWinsock == NULL){
		hWinsock = LoadLibrary("wsock32.dll");
	}


	if (WSAStartup(MAKEWORD(1, 1), &wsadata) < 0){
			MessageBox(NULL, "Winsock initialization failed", "Error", MB_OK);
	}

}

void send_cwd(int sock){
  char buf[MAX_PATH];
  char cmd[MAX_PATH + 9];
  GetCurrentDirectory(sizeof(buf), buf);
  sprintf(cmd, "$cwd=`%s`\n", buf);
  do_write(sock, cmd, strlen(cmd));
}

#ifdef PSHW
#define main _main
#endif

int main(int argc, char** argv)

{
  int i;
  int sock;
  int port = 0;
  char *pname;
  char *command = NULL;
  char pref[10];
  char* cookie;

  pname = argv[0];

  i = 1;
  while (argc > 1){
    char* a = argv[i];
    if (memcmp(a, "-port:", 6) == 0){
      port = atoi(a + 6);
      argc--;
      argv++;
      i++;
    } else if (memcmp(a, "-e", 2) == 0){
      i++;
      command = strdup(argv[i]);
      i++;
      argc -= 2;
      argv += 2;
    } else if (memcmp(a, "-help", 5) == 0){
      printf("Usage:  %s [ -port:NNNN ] [ -e expr | script_file ]\n", pname);
      exit(1);
    } else {
      break;
    }
  }

  if (read_cookie(pref) < 0){
	  MessageBox(NULL, "Cannot read cookie", "Error", MB_OK);
    exit(-1);
  }
  if (port == 0){
    port = ((pref[0] & 0xff) << 8) + (pref[1] & 0xff);
  }

  cookie = pref + 2;

  init_winsock();

  sock = do_connect(HOSTNAME, port);
  if (sock < 0){
	  MessageBox(NULL, "Cannot connect to Pnuts service", "Error", MB_OK);
    exit(-1);
  }

  do_write(sock, cookie, 8); /* send cookie */

  if (argc == 1 && command == NULL){
    int i = -1;
    do_write(sock, (char*)&i, 1);
    send_cwd(sock);
    do_write(sock, "quit()\n", 7);
  } else {
    int i = 0;
    do_write(sock, (char*)&i, 1);
    send_cwd(sock);
  }

  if (command != NULL){
    do_write(sock, command, strlen(command));
    do_write(sock, "\n", 1);
  } else if (argc == 1){ /* read from stdin */
/*    read_stdin(sock);*/

    read_input(stdin, sock);
  } else {
    FILE *file;
    if (set_args(sock, argc, argv) < 0){
      /* ignore */
    }

    if ((file = fopen(argv[1], "r")) == NULL){
      MessageBox(NULL, "Cannot open the script", "Error", MB_OK);
      exit(-1);
    }
    read_input(file, sock);
    fclose(file);
  }

  do_write(sock, "quit()\n", 7);
  receive_output(sock);
  close(sock);
  exit(0);
}

#ifdef PSHW

__declspec(dllexport) char **__initenv;


int WINAPI WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow )
{
	__initenv = _environ;
	return main(__argc, __argv);
}
#endif
